@@ -270,7 +270,6 @@ def _is_on_a_line_by_self(tokens, i):  # type: (List[Token], int) -> bool
     return (
         tokens[i - 2].name == 'NL' and
         tokens[i - 1].name == UNIMPORTANT_WS and
-        tokens[i - 1].src.isspace() and
         tokens[i + 1].name == 'NL'
     )
 
@@ -669,8 +668,54 @@ def _fix_encode_to_binary(tokens, i):  # type: (List[Token], int) -> None
     del tokens[victims]
 
 
-def _fix_tokens(contents_text, py3_plus):  # type: (str, bool) -> str
-    remove_u_prefix = py3_plus or _imports_unicode_literals(contents_text)
+FUTURES = {}  # type: Dict[Tuple[int, ...], Tuple[str, ...]]
+FUTURES[(2, 7)] = ('nested_scopes', 'generators', 'with_statement')
+FUTURES[(3,)] = FUTURES[(3, 6)] = FUTURES[(2, 7)] + (
+    'absolute_import', 'division', 'print_function', 'unicode_literals',
+)
+FUTURES[(3, 7)] = FUTURES[(3, 6)] + ('generator_stop',)
+
+
+def _fix_future_imports(tokens, start, min_version):
+    # type: (List[Token], int, Tuple[int, ...]) -> None
+    i = start + 1
+    while tokens[i].name != 'NAME':
+        i += 1
+    if tokens[i].src != '__future__':
+        return
+
+    found_names = []  # type: List[int]
+    i += 1
+    while tokens[i].name not in {'NEWLINE', 'ENDMARKER'}:
+        if tokens[i].name == 'NAME' and tokens[i].src != 'import':
+            found_names.append(i)
+        i += 1
+    # depending on the version of python, some will not emit NEWLINE('') at the
+    # end of a file which does not end with a newline (for example 2.7.6)
+    if tokens[i].name == 'ENDMARKER':  # pragma: no cover
+        i -= 1
+
+    remove_futures = FUTURES[min_version]
+    to_remove = [x for x in found_names if tokens[x].src in remove_futures]
+    if len(to_remove) == len(found_names):
+        del tokens[start:i + 1]
+    else:
+        for idx in reversed(to_remove):
+            if found_names[0] == idx:  # look forward until next name and del
+                j = idx + 1
+                while tokens[j].name != 'NAME':
+                    j += 1
+                del tokens[idx:j]
+            else:  # look backward for comma and del
+                j = idx
+                while tokens[j].src != ',':
+                    j -= 1
+                del tokens[j:idx + 1]
+
+
+def _fix_tokens(contents_text, min_version):
+    # type: (str, Tuple[int, ...]) -> str
+    remove_u = min_version >= (3,) or _imports_unicode_literals(contents_text)
 
     try:
         tokens = src_to_tokens(contents_text)
@@ -681,7 +726,7 @@ def _fix_tokens(contents_text, py3_plus):  # type: (str, bool) -> str
             tokens[i] = token._replace(src=_fix_long(_fix_octal(token.src)))
         elif token.name == 'STRING':
             tokens[i] = _fix_ur_literals(tokens[i])
-            if remove_u_prefix:
+            if remove_u:
                 tokens[i] = _remove_u_prefix(tokens[i])
             tokens[i] = _fix_escape_sequences(tokens[i])
         elif token.src == '(':
@@ -691,7 +736,7 @@ def _fix_tokens(contents_text, py3_plus):  # type: (str, bool) -> str
         elif token.src == 'encode' and i > 0 and tokens[i - 1].src == '.':
             _fix_encode_to_binary(tokens, i)
         elif (
-                py3_plus and
+                min_version >= (3,) and
                 token.utf8_byte_offset == 0 and
                 token.line < 3 and
                 token.name == 'COMMENT' and
@@ -700,7 +745,9 @@ def _fix_tokens(contents_text, py3_plus):  # type: (str, bool) -> str
             del tokens[i]
             if tokens[i].name == 'NL':  # pragma: no branch (old PY2)
                 del tokens[i]
-    return tokens_to_src(tokens)
+        elif token.src == 'from' and token.utf8_byte_offset == 0:
+            _fix_future_imports(tokens, i, min_version)
+    return tokens_to_src(tokens).lstrip()
 
 
 MAPPING_KEY_RE = re.compile(r'\(([^()]*)\)')
@@ -2123,12 +2170,12 @@ def _fix_file(filename, args):  # type: (str, argparse.Namespace) -> int
         return 1
 
     contents_text = _fix_py2_compatible(contents_text)
-    contents_text = _fix_tokens(contents_text, args.py3_plus)
+    contents_text = _fix_tokens(contents_text, min_version=args.min_version)
     if not args.keep_percent_format:
         contents_text = _fix_percent_format(contents_text)
-    if args.py3_plus:
+    if args.min_version >= (3,):
         contents_text = _fix_py3_plus(contents_text)
-    if args.py36_plus:
+    if args.min_version >= (3, 6):
         contents_text = _fix_fstrings(contents_text)
 
     if filename == '-':
@@ -2149,13 +2196,20 @@ def main(argv=None):  # type: (Optional[Sequence[str]]) -> int
     parser.add_argument('filenames', nargs='*')
     parser.add_argument('--exit-zero-even-if-changed', action='store_true')
     parser.add_argument('--keep-percent-format', action='store_true')
-    parser.add_argument('--py3-plus', '--py3-only', action='store_true')
-    parser.add_argument('--py36-plus', action='store_true')
+    parser.add_argument(
+        '--py3-plus', '--py3-only',
+        action='store_const', dest='min_version', default=(2, 7), const=(3,),
+    )
+    parser.add_argument(
+        '--py36-plus',
+        action='store_const', dest='min_version', const=(3, 6),
+    )
+    parser.add_argument(
+        '--py37-plus',
+        action='store_const', dest='min_version', const=(3, 7),
+    )
     args = parser.parse_args(argv)
 
-    if args.py36_plus:
-        args.py3_plus = True
-
     ret = 0
     for filename in args.filenames:
         ret |= _fix_file(filename, args)
